@@ -30,14 +30,12 @@
 #include <string.h>
 #include <locale.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
+#ifdef HAVE_CODESET
+#include <langinfo.h>
 #endif
 
 #include "gmime-charset-map-private.h"
 #include "gmime-charset.h"
-#include "unicode.h"
-#include "strlib.h"
 
 #ifdef HAVE_ICONV_DETECT_H
 #include "iconv-detect.h"
@@ -65,7 +63,7 @@ struct {
 	{ "utf-8",           "UTF-8"      },
 	
 	/* ANSI_X3.4-1968 is used on some systems and should be
-           treated the same as US-ASCII */
+	   treated the same as US-ASCII */
 	{ "ANSI_X3.4-1968",  NULL         },
 	
 	/* 10646 is a special case, its usually UCS-2 big endian */
@@ -100,6 +98,15 @@ struct {
 static GHashTable *iconv_charsets = NULL;
 static char *locale_charset = NULL;
 
+#ifdef G_THREADS_ENABLED
+static GStaticMutex charset_lock = G_STATIC_MUTEX_INIT;
+#define CHARSET_LOCK()   g_static_mutex_lock (&charset_lock);
+#define CHARSET_UNLOCK() g_static_mutex_unlock (&charset_lock);
+#else
+#define CHARSET_LOCK()
+#define CHARSET_UNLOCK()
+#endif /* G_THREADS_ENABLED */
+
 
 static void
 iconv_charset_free (gpointer key, gpointer value, gpointer user_data)
@@ -108,31 +115,24 @@ iconv_charset_free (gpointer key, gpointer value, gpointer user_data)
 	g_free (value);
 }
 
-
-/**
- * g_mime_charset_shutdown:
- *
- * Frees internal lookup tables created in #g_mime_charset_init().
- **/
-void
+static void
 g_mime_charset_shutdown (void)
 {
 	g_hash_table_foreach (iconv_charsets, iconv_charset_free, NULL);
 	g_hash_table_destroy (iconv_charsets);
 	g_free (locale_charset);
-	iconv_charsets = NULL;
 }
 
 
 /**
- * g_mime_charset_init:
+ * g_mime_charset_map_init:
  *
  * Initializes the locale charset variable for later calls to
  * gmime_charset_locale_name. Only really needs to be called for non-
  * iso-8859-1 locales.
  **/
 void
-g_mime_charset_init (void)
+g_mime_charset_map_init (void)
 {
 	char *charset, *iconv_name, *locale;
 	int i;
@@ -149,6 +149,10 @@ g_mime_charset_init (void)
 		g_hash_table_insert (iconv_charsets, charset, iconv_name);
 	}
 	
+#ifdef HAVE_CODESET
+	locale_charset = g_strdup (nl_langinfo (CODESET));
+	g_strdown (locale_charset);
+#else
 	locale = setlocale (LC_ALL, NULL);
 	
 	if (!locale || !strcmp (locale, "C") || !strcmp (locale, "POSIX")) {
@@ -179,6 +183,9 @@ g_mime_charset_init (void)
 			locale_charset = NULL;
 		}
 	}
+#endif
+	
+	g_atexit (g_mime_charset_shutdown);
 }
 
 
@@ -192,6 +199,11 @@ g_mime_charset_init (void)
 const char *
 g_mime_charset_locale_name (void)
 {
+	CHARSET_LOCK ();
+	if (!iconv_charsets)
+		g_mime_charset_map_init ();
+	CHARSET_UNLOCK ();
+	
 	return locale_charset ? locale_charset : "iso-8859-1";
 }
 
@@ -212,16 +224,19 @@ g_mime_charset_name (const char *charset)
 	if (charset == NULL)
 		return NULL;
 	
-	if (!iconv_charsets)
-		return charset;
-	
-	name = alloca (strlen (charset) + 1);
+	name = g_alloca (strlen (charset) + 1);
 	strcpy (name, charset);
 	g_strdown (name);
 	
+	CHARSET_LOCK ();
+	if (!iconv_charsets)
+		g_mime_charset_map_init ();
+	
 	iconv_name = g_hash_table_lookup (iconv_charsets, name);
-	if (iconv_name)
+	if (iconv_name) {
+		CHARSET_UNLOCK ();
 		return iconv_name;
+	}
 	
 	if (!strncmp (name, "iso", 3)) {
 		int iso, codepage;
@@ -233,10 +248,12 @@ g_mime_charset_name (const char *charset)
 		
 		iso = strtoul (buf, &p, 10);
 		
+		g_assert (p > buf);
+		
 		if (iso == 10646) {
 			/* they all become ICONV_10646 */
 			iconv_name = g_strdup (ICONV_10646);
-		} else if (p > buf) {
+		} else {
 			buf = p;
 			if (*buf == '-' || *buf == '_')
 				buf++;
@@ -257,10 +274,6 @@ g_mime_charset_name (const char *charset)
 				iconv_name = g_strdup_printf (ICONV_ISO_S_FORMAT,
 							      iso, p);
 			}
-		} else {
-			/* p == buf, which probably means we've
-			   encountered an invalid iso charset name */
-			iconv_name = g_strdup (name);
 		}
 	} else if (!strncmp (name, "windows-", 8)) {
 		buf = name + 8;
@@ -281,17 +294,20 @@ g_mime_charset_name (const char *charset)
 	
 	g_hash_table_insert (iconv_charsets, g_strdup (name), iconv_name);
 	
+	CHARSET_UNLOCK ();
+	
 	return iconv_name;
 }
 
 
-typedef struct _Charset {
-	unsigned int mask;
-	unsigned int level;
-} Charset;
-
-static void
-charset_init (Charset *charset)
+/**
+ * g_mime_charset_init:
+ * @charset: charset mask
+ *
+ * Initializes a charset mask structure.
+ **/
+void
+g_mime_charset_init (GMimeCharset *charset)
 {
 	charset->mask = ~0;
 	charset->level = 0;
@@ -299,7 +315,7 @@ charset_init (Charset *charset)
 
 
 /**
- * charset_step:
+ * g_mime_charset_step:
  * @charset:
  * @in: input text buffer (must be in UTF-8)
  * @len: input buffer length
@@ -309,8 +325,8 @@ charset_init (Charset *charset)
  * @charset->mask with the mask for each glyph. This has the effect of
  * limiting what charsets our @charset->mask can match.
  **/
-static void
-charset_step (Charset *charset, const char *in, size_t len)
+void
+g_mime_charset_step (GMimeCharset *charset, const char *in, size_t len)
 {
 	register const char *inptr = in;
 	const char *inend = in + len;
@@ -322,11 +338,11 @@ charset_step (Charset *charset, const char *in, size_t len)
 	
 	while (inptr < inend) {
 		const char *newinptr;
-		unichar c;
+		gunichar c;
 		
-		newinptr = unicode_next_char (inptr);
-		c = unicode_get_char (inptr);
-		if (newinptr == NULL || !unichar_validate (c)) {
+		newinptr = g_utf8_next_char (inptr);
+		c = g_utf8_get_char (inptr);
+		if (newinptr == NULL || !g_unichar_validate (c)) {
 			inptr++;
 			continue;
 		}
@@ -362,8 +378,18 @@ charset_best_mask (unsigned int mask)
 	return "UTF-8";
 }
 
-static const char *
-charset_best_name (Charset *charset)
+
+/**
+ * g_mime_charset_best_name:
+ * @charset: charset mask
+ *
+ * Gets the best charset name based on the charset mask @charset.
+ *
+ * Returns a pointer to a string containing the best charset name that
+ * can represent the charset mask @charset.
+ **/
+const char *
+g_mime_charset_best_name (GMimeCharset *charset)
 {
 	if (charset->level == 1)
 		return "iso-8859-1";
@@ -387,12 +413,12 @@ charset_best_name (Charset *charset)
 const char *
 g_mime_charset_best (const char *in, size_t inlen)
 {
-	Charset charset;
+	GMimeCharset charset;
 	
-	charset_init (&charset);
-	charset_step (&charset, in, inlen);
+	g_mime_charset_init (&charset);
+	g_mime_charset_step (&charset, in, inlen);
 	
-	return charset_best_name (&charset);
+	return g_mime_charset_best_name (&charset);
 }
 
 
