@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#include <string.h>
 #include <sys/types.h>
 #include <regex.h>
 #include <ctype.h>
@@ -36,8 +37,6 @@
 #include "gmime-multipart.h"
 #include "gmime-part.h"
 
-#include "strlib.h"
-
 #ifndef HAVE_ISBLANK
 #define isblank(c) ((c) == ' ' || (c) == '\t')
 #endif /* HAVE_ISBLANK */
@@ -82,6 +81,7 @@ struct _GMimeParserPrivate {
 	unsigned char *inptr;
 	unsigned char *inend;
 	
+	off_t from_offset;
 	GByteArray *from_line;
 	
 	regex_t header_regex;
@@ -174,7 +174,7 @@ header_raw_find (struct _header_raw *headers, const char *name, off_t *offset)
 	
 	h = headers;
 	while (h) {
-		if (!strcasecmp (h->name, name)) {
+		if (!g_strcasecmp (h->name, name)) {
 			if (offset)
 				*offset = h->offset;
 			return h->value;
@@ -282,6 +282,7 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->inptr = priv->inbuf;
 	priv->inend = priv->inbuf;
 	
+	priv->from_offset = -1;
 	priv->from_line = g_byte_array_new ();
 	
 	priv->headerbuf = g_malloc (SCAN_HEAD + 1);
@@ -591,6 +592,7 @@ parser_step_from (GMimeParser *parser)
 			inptr++;
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
+				priv->from_offset = parser_offset (parser, start);
 				g_byte_array_append (priv->from_line, start, len);
 				goto got_from;
 			}
@@ -643,8 +645,16 @@ parser_step_from (GMimeParser *parser)
 	                                                                  \
 	hlen = colon - priv->headerbuf;                                   \
 	                                                                  \
-	header->name = g_strstrip (g_strndup (priv->headerbuf, hlen));    \
-	header->value = g_strstrip (g_strdup (colon + 1));                \
+	header->name = g_strndup (priv->headerbuf, hlen);                 \
+	g_strstrip (header->name);                                        \
+	if (*colon != ':') {                                              \
+		g_warning ("Invalid header: %s", header->name);           \
+		header->value = header->name;                             \
+		header->name = g_strdup ("X-Invalid-Header");             \
+	} else {                                                          \
+		header->value = g_strdup (colon + 1);                     \
+		g_strstrip (header->value);                               \
+	}                                                                 \
 	header->offset = priv->header_start;                              \
 	                                                                  \
 	hend->next = header;                                              \
@@ -666,7 +676,7 @@ parser_step_headers (GMimeParser *parser)
 	register unsigned char *inptr;
 	unsigned char *start, *inend;
 	struct _header_raw *hend;
-	size_t len;
+	size_t len = 0;
 	
 	priv->midline = FALSE;
 	hend = (struct _header_raw *) &priv->headers;
@@ -674,10 +684,11 @@ parser_step_headers (GMimeParser *parser)
 	priv->header_start = parser_offset (parser, NULL);
 	
 	inptr = priv->inptr;
+	inend = priv->inend;
 	
 	do {
 	refill:
-		if (parser_fill (parser) <= 0)
+		if (parser_fill (parser) <= len)
 			break;
 		
 		inptr = priv->inptr;
@@ -697,6 +708,7 @@ parser_step_headers (GMimeParser *parser)
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
 				priv->inptr = start;
+				len = inend - start;
 				goto refill;
 			}
 			
@@ -723,6 +735,7 @@ parser_step_headers (GMimeParser *parser)
 		}
 		
 		priv->inptr = inptr;
+		len = inend - inptr;
 	} while (1);
 	
 	inptr = priv->inptr;
@@ -980,6 +993,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	wrapper = g_mime_data_wrapper_new_with_stream (stream, encoding);
 	g_mime_part_set_content_object (mime_part, wrapper);
 	g_mime_stream_unref (stream);
+	g_object_unref (wrapper);
 }
 
 static void
@@ -1309,3 +1323,28 @@ g_mime_parser_get_from (GMimeParser *parser)
 	
 	return NULL;
 }
+
+
+/**
+ * g_mime_parser_get_from_offset:
+ * @parser: MIME parser object
+ *
+ * Gets the offset of the most recently parsed mbox-style From-line
+ * (gotten from #g_mime_parser_construct_message).
+ *
+ * Returns the offset of the most recently parsed mbox-style From-line
+ * or -1 on error.
+ **/
+off_t
+g_mime_parser_get_from_offset (GMimeParser *parser)
+{
+	struct _GMimeParserPrivate *priv;
+	
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
+	
+	priv = parser->priv;
+	if (!priv->scan_from)
+		return -1;
+	
+	return priv->from_offset;
+}
