@@ -36,17 +36,14 @@
 
 
 #define w(x) x
-
-
-extern int gmime_interfaces_utf8;
-
+#define d(x)
 
 /**
  * internet_address_new:
  *
- * Creates a new InternetAddress object
+ * Creates a new #InternetAddress object
  * 
- * Returns a new Internet Address object.
+ * Returns a new #InternetAddress object.
  **/
 InternetAddress *
 internet_address_new ()
@@ -65,9 +62,9 @@ internet_address_new ()
 
 /**
  * internet_address_destroy:
- * @ia: InternetAddress object to destroy
+ * @ia: internet address
  * 
- * Destroy the InternetAddress object pointed to by @ia.
+ * Destroy the #InternetAddress object pointed to by @ia.
  **/
 static void
 internet_address_destroy (InternetAddress *ia)
@@ -88,7 +85,7 @@ internet_address_destroy (InternetAddress *ia)
 
 /**
  * internet_address_ref:
- * @ia: address
+ * @ia: internet address
  *
  * Ref's the internet address.
  **/
@@ -101,7 +98,7 @@ internet_address_ref (InternetAddress *ia)
 
 /**
  * internet_address_unref:
- * @ia: address
+ * @ia: internet address
  *
  * Unref's the internet address.
  **/
@@ -121,10 +118,10 @@ internet_address_unref (InternetAddress *ia)
  * @name: person's name
  * @addr: person's address
  *
- * Creates a new InternetAddress object with name @name and address
+ * Creates a new #InternetAddress object with name @name and address
  * @addr.
  * 
- * Returns a new Internet Address object.
+ * Returns a new #InternetAddress object.
  **/
 InternetAddress *
 internet_address_new_name (const char *name, const char *addr)
@@ -149,9 +146,9 @@ internet_address_new_name (const char *name, const char *addr)
  * internet_address_new_group:
  * @name: group name
  *
- * Creates a new InternetAddress object with group name @name.
+ * Creates a new #InternetAddress object with group name @name.
  * 
- * Returns a new Internet Address object.
+ * Returns a new #InternetAddress object.
  **/
 InternetAddress *
 internet_address_new_group (const char *name)
@@ -438,11 +435,11 @@ internet_address_list_to_string_internal (InternetAddressList *list, gboolean en
  * @ia: Internet Address object
  * @encode: TRUE if the address should be rfc2047 encoded
  *
- * Allocates a string containing the contents of the InternetAddress
+ * Allocates a string containing the contents of the #InternetAddress
  * object.
  * 
- * Returns the InternetAddress object as an allocated string in rfc822
- * format.
+ * Returns the #InternetAddress object as an allocated string in
+ * rfc822 format.
  **/
 char *
 internet_address_to_string (InternetAddress *ia, gboolean encode)
@@ -481,7 +478,7 @@ internet_address_to_string (InternetAddress *ia, gboolean encode)
 /**
  * internet_address_list_to_string:
  * @list: list of internet addresses
- * @encode: TRUE if the address should be rfc2047 encoded
+ * @encode: %TRUE if the address should be rfc2047 encoded
  *
  * Allocates a string buffer containing the rfc822 formatted addresses
  * in @list.
@@ -758,7 +755,7 @@ decode_mailbox (const char **in)
 		if (name)
 			g_string_free (name, TRUE);
 		g_string_free (addr, TRUE);
-		*in = inptr + 1;
+		*in = inptr;
 		
 		return NULL;
 	}
@@ -817,10 +814,15 @@ decode_mailbox (const char **in)
 				const char *cend;
 				
 				/* find the end of the comment */
-				for (cend = inptr - 1; cend > comment && is_lwsp (*cend); cend--);
+				cend = inptr - 1;
+				while (cend > comment && is_lwsp (*cend))
+					cend--;
+				
 				if (*cend == ')')
 					cend--;
+				
 				comment = g_strndup (comment + 1, cend - comment);
+				g_strstrip (comment);
 				
 				name = g_string_new (comment);
 				g_free (comment);
@@ -831,10 +833,12 @@ decode_mailbox (const char **in)
 	*in = inptr;
 	
 	if (addr->len) {
-		if (gmime_interfaces_utf8 && name && g_mime_utils_text_is_8bit (name->str, name->len)) {
-			/* A (broken) mailer has sent us an unencoded 8bit value.
-			 * Attempt to save it by assuming it's in the user's
-			 * locale and converting to UTF-8 */
+		if (name && !g_utf8_validate (name->str, -1, NULL)) {
+			/* A (broken) mailer has sent us an unencoded
+			 * 8bit value (and it doesn't seem to be valid
+			 * UTF-8 either).  Attempt to save it by
+			 * assuming it's in the user's locale and
+			 * converting to UTF-8 */
 			char *buf;
 			
 			buf = g_mime_iconv_locale_to_utf8 (name->str);
@@ -843,8 +847,8 @@ decode_mailbox (const char **in)
 				g_string_append (name, buf);
 				g_free (buf);
 			} else {
-				(g_warning ("Failed to convert \"%s\" to UTF-8: %s",
-					    name->str, g_strerror (errno)));
+				d(g_warning ("Failed to convert \"%s\" to UTF-8: %s",
+					     name->str, g_strerror (errno)));
 			}
 		}
 		
